//1080p 2bpp Tile Driver Test in FlexProp C
//Raymond Allen 2020  MIT license

#include "Platform.h"  //Most of the P2 board settings here

// the 2bpp tile driver with VGA output
struct __using("1080p_TileDriver_8b.spin2") vga; //RJA:  Visual Studio will show an error here, but it's OK

// access to the cogless graphics driver inside the vga driver
struct __using("Graphics3g.spin2") gr; //RJA:  Visual Studio will show an error here, but it's OK

//GarryJ's one cog USB Keyboard/Mouse driver
struct __using("KbM_Interface_rja2f.spin2") kbm; //RJA:  Visual Studio will show an error here, but it's OK

#include "stdbool.h"
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>






//1080p Screen parameters for tile driver
#define Cols  (120)       //characters are 16 pixels wide
#define Rows  (68)        //16 pixels tall   (two 16-pixel rows per character) (only 1/2 of last row visible)
//Graphics buffer size
#define xtiles (64)
#define ytiles (6)



#include "Forms.h"  //Structures and data that define forms
#include "Form1.h"  //Data and functions that define Form #1

void StartApplication()
{//Do things custom to this application

    //Start up cogged graphics
    gr2.start(Graphics_Cog);  //VS shows error, but it's OK

    //Have smartpins generate example digital data on startup pins
    InitSmartPins();

    InitSlider(&Form1Slider1); // set form slider position to match initial value
    InitSlider(&Form1Slider2); // set form slider position to match initial value

    //Start up DDS to make analog traces to look at
    dds.Start(ddsCog);  //VS shows error, but it's OK

    //Start up digitizer
    int mode = 0;
    dig.Start(digCog);  //VS shows error, but it's OK
    int i = 44; //smartpin looking at NCO output from P45
    Form1NumberEdit1.value = i; //set trigger pin value in number edit control

    dig.SetTriggerPin(i);
    dig.SetTriggerMode(0); //'0=digital, 1=analog
    FormOption* pFO = &Form1Option3;
    pFO->bSelected = true; // digital trigger mode on
    pFO = &Form1Option4;
    pFO->bSelected = false;// analog trigger mode off
    EnableObject((FormObject*)&Form1Slider2, false);
    mode = 1; //1=enabled
    dig.SetMode(mode);

    //Initialize Horizontal clocks/point display and settings
    HandleWaitMouseUpSpecialForm1((FormObject*)&Form1Dial1);
    dig.SetWait(0);//time between points

    //FormSlider* pFS = &Form1Slider2;
    //pFS->    , 0); //disable trigger level slider

    //'Start up digitizer
    //mode:=0 'disabled
    //dig.Start(digCog)
    //i:=44'54 '54=smartpin looking at NCO output from P55
    //SetNumberEditValue(@Form1Number1,i) 'set trigger pin value in number edit control
    //dig.SetTriggerPin(i)'A=first digital channel, A1=first analog channel
    //dig.SetTriggerMode(0) '0=digital, 1=analog
    //SetOptionValue(@Form1Option3,1)'digital trigger mode on
    //SetOptionValue(@Form1Option4,0)'analog trigger mode off
    //SetEnabled(@Form1Slider2,0) 'disable trigger level slider
    //mode:=1 '1=enabled
    //dig.SetMode(mode)
}

void InitSmartPins()
{//Have some smartpins generate a square wave and have neighboring pins look at it to show on scope

    //NOTE:  See Spin2 docs for values of things like P_PLUS1_A and PINSTART
    _pinstart(45, P_NCO_FREQ + P_OE, 0x80, 0x20000000); //NCO on Pin 45
    _pinstart(44, P_PLUS1_A, 0, 0); //Pin 44 to snoop on the output of Pin 45

    _pinstart(47, P_NCO_FREQ + P_OE, 0x80, 0x07000000); //NCO on Pin 47
    _pinstart(46, P_PLUS1_A, 0, 0); //Pin 46 to snoop on the output of Pin 47

    _pinstart(49, P_NCO_FREQ + P_OE, 0x80, 0x03000000); //NCO on Pin 49
    _pinstart(48, P_PLUS1_A, 0, 0); //Pin 48 to snoop on the output of Pin 49

    //By default, we have D1 on P44 and set to look at D2 on P45 doing NCO
    RPIN[2]=(P_NCO_FREQ + P_OE); //NCO on Pin 45)
    RPIN[3]=(P_NCO_FREQ + P_OE); //NCO on Pin 47
    XPIN[3] = 0x80;
    YPIN[3] = 0x20000000;

    //By default, we have D3 on P46 and set to look at D4 on P47 doing NCO
    RPIN[4] = (P_NCO_FREQ + P_OE); //NCO on Pin 45)
    RPIN[5] = (P_NCO_FREQ + P_OE); //NCO on Pin 47
    XPIN[5] = 0x80;
    YPIN[5] = 0x07000000;

    //By default, we have D5 on P48 and set to look at D6 on P49 doing NCO
    RPIN[6] = (P_NCO_FREQ + P_OE); //NCO on Pin 45)
    RPIN[7] = (P_NCO_FREQ + P_OE); //NCO on Pin 47
    XPIN[7] = 0x80;
    YPIN[7] = 0x03000000;



    //The analog channels are set to scope mode by the digitizer
    RPIN[0] = scpmode;
    XPIN[0] = scp_x;
    RPIN[1] = scpmode;
    XPIN[1] = scp_x;

}





int main()
{    //set the clock using setting in Platform.h
    //Regular way doesn't currently seem to work for 297 MHz, so getting settings from VGA driver
    //_clkset(_SETFREQ, _CLOCKFREQ);
    _clkset(vga.GetClockMode(), vga.GetClockFreq());

    //check clock freq
    int cf = _clockfreq();
    printf("clock freq= %d\n", cf);

    printf("Starting Graphics...\n");
    int pCommand = gr.start();
    int pFont = gr.GetFont();

    FormForm* pForm = &Form1;

    printf("Starting Mouse...\n");
    MouseStatus.MaxX = 1920-1;
    MouseStatus.MaxY = 1080-1;
    MouseStatus.x = 100;
    MouseStatus.y = 100;
    kbm.StartMouse(&MouseStatus);

    printf("Starting VGA...\n");
    vga.start(&MouseStatus, pCommand, pFont, & DebugLong);
    //vga.main();
    //vga.test11();
    //printf("VGA Started and returns %d\n",cog);

    vga.cls(vga.white);

    StartApplication();//Do things custom to this application

    pCurrentForm = &Form1;
    DrawForm(pCurrentForm);

    
       
    while (true)
    {//app interface loop
        //kb.clearkeys           //clear keyboard buffer(only accept keys while waiting on mouse down)
        WaitMouseDown();          //wait for user to push mouse down
        HandleMouseDown();
        WaitMouseUp();            //then wait for user to let mouse go
        HandleMouseUp();
    } //stop here
}
