////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Form Definitions
#define MaxFormText (1920/16) //make it width of screen in tiles
#define MaxFormObjects 100



enum FormObjectTypes { FormType = 0, ButtonType = 1, TextType, GroupType, LedType, OptionType, CheckType, LedArrayType, SliderType, GraphicsAreaType, GraphicsObjectType, ProgressType, DialType, NumberEditType };

typedef struct
{
    int type;
    int top;
    int left;
    int width;
    int height;
    int data;
    int status;
} FormObject;

typedef struct {
    FormObject* p[MaxFormObjects];
} FormObjectList;

typedef   void (*FormOperation)(FormObject* pObject);

typedef struct
{//Form Structure
    int type;               //Object type form==0
    int top;                //Top tile#
    int left;               //Left tile#
    int width;              //width in tiles
    int height;             //height in tiles
    int data;               //user data
    int status;             //status: Lowest bit of status is invisible flag, Second lowest bit is disabled flag
    int bordercolor;        //Border color
    int fillcolor;          //fill color
    char* pTitleText;           //title bar text
    int textcolor;          //color of title bar text
    FormObject* pParent;    //pointer to parent form (or 0 if this is top form)
    FormObjectList* pObjectList;   //pointer to NULL terminated list of form objects
    FormOperation OnFormOpen;  //Function to call when opening this form
    FormOperation OnFormClose;  //Function to call when closing this form
    FormOperation OnIdle;  //Function to call when idle
} FormForm;


typedef struct
{
    int type;               //Object type button==1
    int top;                //Top tile #
    int left;               //Left tile #
    int width;              //Width in tiles
    int height;             //Height in tiles
    int data;               //user data
    int status;             //status: Lowest bit of status is invisible flag, Second lowest bit is disabled flag
    int textforecolor;      //Text foreground color
    int textbackcolor;      //Text background color  (usually same as inside color)
    int bordercolor;        //border color
    int highlightedbordercolor; //border color when highlighted
    int outsidecolor;       //color outside button (usually same as form background)
    int insidecolor;        //color inside button
    int hotkeyposition;     //Hot key position (0 for none)
    int hotkeycolor;        //Color for hot key character
    int style;              //Style:  0=square, 1=round
    char* pText;            //Button text
    FormOperation OnClick;  //Function to call when button clicked
} FormButton;

typedef struct
{
    int type;               //Object type text==2
    int top;                //Top tile #
    int left;               //Left tile #
    int width;              //Width in tiles
    int height;             //Height in tiles
    int data;               //user data
    int status;             //status: Lowest bit of status is invisible flag, Second lowest bit is disabled flag
    int textforecolor;      //Text foreground color
    int textbackcolor;      //Text background color  (usually same as inside color)
    char* pText;            //text
} FormText;


typedef struct
{
    int type;               //Object type group=3
    int top;                //Top tile #
    int left;               //Left tile #
    int width;              //Width in tiles
    int height;             //Height in tiles
    int data;               //user data
    int status;             //status: Lowest bit of status is invisible flag, Second lowest bit is disabled flag
    int textforecolor;      //Text foreground color
    int bordercolor;        //Border color of group
    char* pText;             //text
} FormGroup;

typedef struct
{
    int type;               //Object type option=5
    int top;                //Top tile #
    int left;               //Left tile #
    int width;              //Width in tiles
    int height;             //Height in tiles
    int data;               //user data
    int status;             //status: Lowest bit of status is invisible flag, Second lowest bit is disabled flag
    int textforecolor;      //Text foreground color
    int outercolor;         //outer color
    int innercolor;         //inner color
    int corecolor;          //core color
    int group;              //group#  0=no group
    FormOperation OnSelected; //Function to call when Number Changed
    int bSelected;         //value, 0=not selected, 1=selected
    char* pText;            //text
} FormOption;

typedef struct
{
    int type;               //Object type check=6
    int top;                //Top tile #
    int left;               //Left tile #
    int width;              //Width in tiles
    int height;             //Height in tiles
    int data;               //user data
    int status;             //status: Lowest bit of status is invisible flag, Second lowest bit is disabled flag
    int textforecolor;      //Text foreground color
    int outercolor;         //outer color
    int innercolor;         //inner color
    int checkcolor;          //check color
    int bSelected;         //value, 0=not selected, 1=selected
    char* pText;            //text
} FormCheck;

typedef struct
{
    int type;               //0 Object type check=6
    int top;                //1 Top tile #
    int left;               //2 Left tile #
    int width;              //3 Width in tiles
    int height;             //4 Height in tiles
    int data;               //5 user data
    int status;             //6 status
    int backcolor;          //7 background color
    int rangecolor;         //8 range bar and tic color
    int outsidecolor;       //9 Outside of position Line
    int centercolor;        //10 Center of position line
    int max;                //11 max value
    int min;                //12 min value
    int value;              //13 current value
    int offset;             //14 current pixel offset
    int tics;               //15 number of tics
    int style;              //16 slider style
    int* pData;              //17 pointer to storage for graphics pixels
    FormOperation OnMouseDown; //18 Function to call when mouse is down on this object
} FormSlider;

typedef struct
{
    int type;               //0 Object type Dial=10
    int top;                //1 Top tile #
    int left;               //2 Left tile #
    int width;              //3 Width in tiles
    int height;             //4 Height in tiles
    int data;               //5 user data
    int status;             //6 status
    int backcolor;          //7 background color
    int indicatorcolor;     //8 dial indicator color
    int ticcolor;           //9 dial tic color
    int dialcolor;          //10 dial color
    int max;                //11 max value
    int min;                //12 min value
    int value;              //13 current value
    int angle;              //14 Current angle (degrees*10): 0=up, -900=left, 900=right
    int startangle;         //15 starting angle
    int endangle;           //16 ending angle
    int tics;               //17 number of tics (actually, number of subdivisions)
    int style;              //18 slider style
    int* pData;              //19 pointer to storage for graphics pixels
    FormOperation OnMouseDown; //20 Function to call when mouse is down on this object
} FormDial;

typedef struct
{
    int type;               //Object type check=6
    int top;                //Top tile #
    int left;               //Left tile #
    int width;              //Width in tiles
    int height;             //Height in tiles
    int data;               //user data
    int status;             //status: Lowest bit of status is invisible flag, Second lowest bit is disabled flag
    int textforecolor;      //Text foreground color
    int textbackcolor;      //Text background color
    int buttoncolor0;       //button color 0
    int buttoncolor1;       //button color 1
    int buttoncolor2;       //button color 2
    int buttoncolor3;       //button color 3
    int value;              //value
    int min;                //min. value
    int max;                //max. vlaue
    int digits;             //#digits
    int base;               //base (10,2,16)
    FormOperation OnChanged; //Function to call when Number Changed
    bool bShowSign;         //Show sign?  //FlexSpin Bug!  Change this to int and it works!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    bool bShowButtons;      //show buttons?
    bool bOnlyClicked;      //Affect only clicked digit?
} FormNumberEdit;

typedef struct
{
    int type;               //Object type check=7
    int top;                //Top tile #
    int left;               //Left tile #
    int width;              //Width in tiles
    int height;             //Height in tiles
    int data;               //user data
    int status;             //status: Lowest bit of status is invisible flag, Second lowest bit is disabled flag
    int color0;             //color #0
    int color1;             //color #1
    int color2;             //color #2
    int color3;             //color #3
    int xOrigin;            //x origin pixel
    int yOrigin;            //y origin pixel
    int* pData;             //pointer to pixel data
    FormOperation OnClick;  //Function to call when clicked
} FormGraphicsArea;

enum FormColors {
    Black = 0, Stratos, NavyBlue, NavyBlue2, DarkBlue, Blue, DeepFir, DeepTeal,
    MidnightBlue, Smalt, InternationalKleinBlue, BlueRibbon, Camarone, FunGreen, BlueStone,
    BahamaBlue, ScienceBlue, BlueRibbon2, JapaneseLaurel, FunGreen2, GreenHaze, PersianGreen,
    PacificBlue, AzureRadiance, Green, Malachite, Jade, CaribbeanGreen, RobinsEggBlue,
    Aqua, Green2, Green3, SpringGreen, SpringGreen2, BrightTurquoise, Cyan, Temptress,
    Barossa, Christalle, PigmentIndigo, DarkBlue2, Blue2, Madras, MineShaft, MineShaft2,
    Sapphire, GovernorBay, Blue3, Blue4, Killarney, Casal, Azure, Azure2, DodgerBlue, Limeade,
    Apple, Apple2, Lochinvar, CuriousBlue, DodgerBlue2, Harlequin, Apple3, Emerald, Shamrock,
    Turquoise, DodgerBlue3, Harlequin2, Harlequin3, ScreaminGreen, SpringGreen3, BrightTurquoise2,
    Aqua2, Lonestar, TyrianPurple, Pompadour, Purple, Purple2, ElectricViolet, NutmegWoodFinish,
    Buccaneer, Cosmic, RoyalPurple, PurpleHeart, ElectricViolet2, VerdunGreen, CostaDelSol, DoveGray, Kimberly,
    BlueMarguerite, CornflowerBlue, Limeade2, Apple4, Highland, Patina, Danube, Malibu, Limeade3,
    Atlantis, Mantis, Emerald2, Downy, Malibu2, BrightGreen, BrightGreen2, ScreaminGreen2,
    ScreaminGreen3, Aquamarine, Aquamarine2, RedBerry, Paprika, FreshEggplant, Flirt, ElectricViolet3,
    ElectricViolet4, Oregon, Stiletto, Rouge, Plum, PurpleHeart2, ElectricViolet5, ChelseaGem, PottersClay, CopperRose,
    Strikemaster, Amethyst, Heliotrope, Olive, Sycamore, Avocado, Avocado2, BlueBell, Melrose,
    Pistachio, Atlantis2, WildWillow, DeYork, Sinbad, Anakiwa, Chartreuse, GreenYellow,
    ScreaminGreen4, MintGreen, Aquamarine3, Anakiwa2, GuardsmanRed, Monza, Lipstick,
    HollywoodCerise, PurplePizzazz, ElectricViolet6, Grenadier, Grenadier2, Hibiscus,
    MediumRedViolet, FuchsiaPink, Heliotrope2, Tenn, Tuscany, ChestnutRose, ChestnutRose2,
    FuchsiaPink2, Heliotrope3, BuddhaGold, HokeyPokey, AntiqueBrass, Eunry, Lilac, Mauve,
    RioGrande, EarlsGreen, Laser, PineGlade, PineGlade2, Periwinkle, ElectricLime, GreenYellow2,
    Canary, Reef, SnowyMint, Onahau, Red, TorchRed, Rose, HollywoodCerise2, HollywoodCerise3,
    Magenta, Scarlet, RedOrange, RadicalRed, WildStrawberry, RazzleDazzleRose, RazzleDazzleRose2,
    BlazeOrange, OutrageousOrange, Bittersweet, HotPink, HotPink2, HotPink3, OrangePeel,
    NeonCarrot, AtomicTangerine, MonaLisa, CarnationPink, LavenderRose, Supernova, Sunglow,
    GoldenTainoi, PeachOrange, PeachOrange2, PinkLace, Yellow, GoldenFizz, LaserLemon,
    PaleCanary, Cream, White, CodGray, MineShaft3, Tundora, Boulder, Gray, SilverChalice,
    Silver, Alto, Gallery
};


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Begining of Form Interface Code
bool bLeftButtonDown = false, bRightButtonDown = false;
bool bMouseMoved;
bool bNeedSync;

/* define a struct, and a pointer-to-struct */
FormObject* pMouseOverObject;
FormObject* pClickedOnObject;
FormForm* pCurrentForm;

typedef struct
{
    int x;
    int y;
    int MaxX;
    int MaxY;
    int buttons;
} MouseInfo;

MouseInfo MouseStatus = { 0,0,0,0 };
MouseInfo OldMouseStatus = { 0,0,0,0 };

int DebugLong;

bool bInvisible, bDisabled;
int oLeft, oTop; //absolute position

void GetObject(FormObject* p)
{//get absolute object position on screen (and some other things)
    bInvisible = p->status & 0b01;//Lowest bit of status is invisible flag
    bDisabled = p->status & 0b10;//Second lowest bit is disabled flag
    oTop = p->top + pCurrentForm->top;
    oLeft = p->left + pCurrentForm->left;
}

void EnableObject(FormObject* p, bool bEnabled)
{
    if (bEnabled)
    {//clear the bDisabled bit
        p->status = p->status & (~0b10);//Second lowest bit is disabled flag
    }
    else
    {//set the bDisabled bit
        p->status = p->status | 0b10;//Second lowest bit is disabled flag
    }
}

void DrawCenteredText(int top, int left, int width, int fore, int back, char* pstr)
{//Draw centered text (like form title)
    vga.setrow(top);
    vga.setforecolor(fore);
    vga.setbackcolor(back);
    //draw centered text
    vga.SetCol(left + 1 + (width - 1 - strlen(pstr)) / 2);
    vga.print_string(pstr);
}

void FormBorder(int top, int left, int height, int width, int clr)
{//Draws border of a form
    int right = left + width - 1;
    int bottom = top + height + 1;
    //top row
    vga.setboxptr(top * vga.cols + left);
    vga.boxchr2(0x0, clr);
    for (int i = 1; i <= (width - 2); i++)
    {
        vga.boxchr2(0xC, clr);
    }
    vga.boxchr2(0x8, clr);
    //center rows
    for (int i = 1; i <= height - 2; i++)
    {
        int p = (top + i) * vga.cols + left;
        vga.setboxptr(p);
        vga.boxchr2(0xA, clr);
        p += width - 1;
        vga.setboxptr(p);
        vga.boxchr2(0xB, clr);
    }
    //bottom row
    vga.setboxptr((top + height - 1) * vga.cols + left);
    vga.boxchr2(0x1, clr);
    for (int i = 1; i <= width - 2; i++)
    {
        vga.boxchr2(0xD, clr);
    }
    vga.boxchr2(0x9, clr);
}



void DrawBoxBorder(int top, int left, int height, int width, int clr)
{//Draw a custom box border using custom characters tacked on to ROM FONT
 //With tile data right after ROM font and then treat as extra characters
    int right = left + width - 1;
    int bottom = top + height + 1;
    //top row
    vga.setboxptr(top * vga.cols + left);
    vga.boxchr2(268, clr);
    for (int i = 1; i <= (width - 2); i++)
    {
        vga.boxchr2(269, clr);
    }
    vga.boxchr2(270, clr);
    //center rows
    for (int i = 1; i <= height - 2; i++)
    {
        int p = (top + i) * vga.cols + left;
        vga.setboxptr(p);
        vga.boxchr2(271, clr);
        p += width - 1;
        vga.setboxptr(p);
        vga.boxchr2(273, clr);
    }
    //bottom row
    vga.setboxptr((top + height - 1) * vga.cols + left);
    vga.boxchr2(274, clr);
    for (int i = 1; i <= width - 2; i++)
    {
        vga.boxchr2(275, clr);
    }
    vga.boxchr2(276, clr);
}

void DrawBoxBorderRounded(int top, int left, int height, int width, int clr)
{//Draw a custom box border using custom characters tacked on to ROM FONT
 //With tile data right after ROM font and then treat as extra characters
    int right = left + width - 1;
    int bottom = top + height + 1;
    //top row
    vga.setboxptr(top * vga.cols + left);
    vga.boxchr2(268 + 9, clr);
    for (int i = 1; i <= (width - 2); i++)
    {
        vga.boxchr2(269 + 9, clr);
    }
    vga.boxchr2(270 + 9, clr);
    //center rows
    for (int i = 1; i <= height - 2; i++)
    {
        int p = (top + i) * vga.cols + left;
        vga.setboxptr(p);
        vga.boxchr2(271 + 9, clr);
        p += width - 1;
        vga.setboxptr(p);
        vga.boxchr2(273 + 9, clr);
    }
    //bottom row
    vga.setboxptr((top + height - 1) * vga.cols + left);
    vga.boxchr2(274 + 9, clr);
    for (int i = 1; i <= width - 2; i++)
    {
        vga.boxchr2(275 + 9, clr);
    }
    vga.boxchr2(276 + 9, clr);
}

void DrawText(FormText* p)
{//Draw text
    GetObject((FormObject*)p);
    vga.setBackColor(p->textbackcolor);
    vga.setForeColor(p->textforecolor);
    vga.setCol(oLeft);
    vga.setRow(oTop);
    vga.print_string2(p->pText);
}

void DrawGroupBox(FormGroup* p)
{//Draw Group Box
    GetObject((FormObject*)p);
    int clrset = vga.MakeColorSet(pCurrentForm->fillcolor, pCurrentForm->fillcolor, p->bordercolor, p->bordercolor);
    //draw the box
    FormBorder(p->top + 1 + pCurrentForm->top, p->left + pCurrentForm->left, p->height, p->width, clrset);
    //draw any text using form's color
    vga.setBackColor(pCurrentForm->fillcolor);
    vga.setForeColor(p->textforecolor);
    vga.setCol(oLeft + 2);
    vga.setRow(oTop);
    vga.print(32);
    vga.print_string(p->pText);
    vga.print(32);
}

void DrawOption(FormOption* p)
{//Draw Option
    GetObject((FormObject*)p);
    int clrset;
    if (p->bSelected)
        clrset = vga.MakeColorSet(pCurrentForm->fillcolor, p->innercolor, p->outercolor, p->corecolor);
    else
        clrset = vga.MakeColorSet(pCurrentForm->fillcolor, p->innercolor, p->outercolor, p->innercolor);
    vga.setBackColor(pCurrentForm->fillcolor);
    vga.setForeColor(p->textforecolor);
    vga.setCol(oLeft + 2);
    vga.setRow(oTop);
    vga.print_string(p->pText);
    vga.SetBoxPtr(oTop * Cols + oLeft);
    vga.boxchr2(260, clrset);
    vga.boxchr2(261, clrset);
    vga.SetBoxPtr((oTop + 1) * Cols + oLeft);
    vga.boxchr2(262, clrset);
    vga.boxchr2(263, clrset);
}

void DrawCheck(FormCheck* p)
{//Draw check
    GetObject((FormObject*)p);
    int clrset;
    if (!p->bSelected)
        clrset = vga.MakeColorSet(pCurrentForm->fillcolor, p->innercolor, p->checkcolor, p->checkcolor);
    else
        clrset = vga.MakeColorSet(pCurrentForm->fillcolor, p->innercolor, p->innercolor, p->checkcolor);
    //draw the text
    vga.setBackColor(pCurrentForm->fillcolor);
    vga.setForeColor(p->textforecolor);
    vga.setCol(oLeft + 2);
    vga.setRow(oTop);
    vga.print_string(p->pText);
    //draw the CheckBox
    vga.SetBoxPtr(oTop * Cols + oLeft);
    vga.boxchr2(256, clrset);
    vga.boxchr2(257, clrset);
    vga.SetBoxPtr((oTop + 1) * Cols + oLeft);
    vga.boxchr2(258, clrset);
    vga.boxchr2(259, clrset);
}



void DrawHorizontalSlider(FormSlider* p)
{//Draw Horizontal Slider
    GetObject((FormObject*)p);
    if (bInvisible)
        return;


    //Little bit of dilemna here...
    //  p->pData is int pointer to int array for pixel storage, but if we leave as int pointer the compiler does the byte pointer math wrong below
    //  So, converting to int so that math below work just like in Spin
    //  The other approach would be to leave as int*, but remove the "*4" in the *pDat assignement below
    int pBuffer = (int)p->pData;

    int clrset;
    clrset = vga.MakeColorSet(p->backcolor, p->rangecolor, p->outsidecolor, p->centercolor);
    //Point tiles to reserved graphics data at p1 and give color
    int* gri = vga.GetTiles();
    int* grj = vga.GetTileColors();
    int grk = 0;
    int* pDat;
    for (int dx = 0; dx < p->width; dx++)
        for (int dy = 0; dy < p->height; dy++)
        {//loop over tiles, setting tile pixel pointers and tile color values
            pDat = (gri + (dy + oTop) * 120 + dx + oLeft);  //This is the address of a tile's pointer to tile pixel data
            *pDat = pBuffer + grk * 16 * 4;  //this is the address of the tile pixel data we want this tile to point to 
            pDat = (grj + (dy + oTop) * 120 + dx + oLeft);
            *pDat = clrset;
            grk++;
        }
    //start and setup graphics in shared buffer (Note:  We could have used p->pData directly, but this way reduces flicker)
    gr.setup(p->width, p->height, 0, 0, bitmap_base);

    //(0,0) is bottom left corner and positive is up and right
    //container size in pixels
    int w = p->width;
    int h = p->height;
    w <<= 4;
    h <<= 4;
    //Test graphics
    gr.clear();


    //draw horizontal bar
    gr.colorwidth(1, 5);
    gr.plot(6, h / 2);
    gr.line(w - 6, h / 2);
    //Draw tics
    int dy = p->tics; //number of tics
    int dx;
    if (dy > 0)
    {
        dx = 10000 * (w - 12) / (dy - 1); //space between tics
        gr.colorwidth(1, 0);
        for (int i = 0; i < dy; i++)
        {
            gr.plot(6 + i * dx / 10000, (h / 2) - 10);
            gr.line(6 + i * dx / 10000, (h / 2) - 16);
        }
    }
    if (!bDisabled) //Enabled?
    {//draw Vertical Indicator
        int sx = p->offset;
        gr.colorwidth(2, 6);
        gr.plot(sx, h / 2 - 10);
        gr.line(sx, h / 2 + 10);
        gr.colorwidth(3, 2);
        gr.plot(sx, h / 2 - 8);
        gr.line(sx, h / 2 + 8);
    }
    //copy buffer to slider pixel storage
    gr.copy(pBuffer);
}

void DrawVerticalSlider(FormSlider* p)
{//Draw Vertical Slider
    GetObject((FormObject*)p);
    if (bInvisible)
        return;

    //Little bit of dilemna here...
    //  p->pData is int pointer to int array for pixel storage, but if we leave as int pointer the compiler does the byte pointer math wrong below
    //  So, converting to int so that math below work just like in Spin
    //  The other approach would be to leave as int*, but remove the "*4" in the *pDat assignement below
    int pBuffer = (int)p->pData;

    int clrset;
    clrset = vga.MakeColorSet(p->backcolor, p->rangecolor, p->outsidecolor, p->centercolor);
    //Point tiles to reserved graphics data at p1 and give color
    int* gri = vga.GetTiles();
    int* grj = vga.GetTileColors();
    int grk = 0;
    int* pDat;
    for (int dx = 0; dx < p->width; dx++)
        for (int dy = 0; dy < p->height; dy++)
        {//loop over tiles, setting tile pixel pointers and tile color values
            pDat = (gri + (dy + oTop) * 120 + dx + oLeft);  //This is the address of a tile's pointer to tile pixel data
            *pDat = pBuffer + grk * 16 * 4;  //this is the address of the tile pixel data we want this tile to point to 
            pDat = (grj + (dy + oTop) * 120 + dx + oLeft);
            *pDat = clrset;
            grk++;
        }
    //start and setup graphics in shared buffer (Note:  We could have used p->pData directly, but this way reduces flicker)
    gr.setup(p->width, p->height, 0, 0, bitmap_base);

    //(0,0) is bottom left corner and positive is up and right
    //container size in pixels
    int w = p->width;
    int h = p->height;
    w <<= 4;
    h <<= 4;
    //erase graphics area
    gr.clear();


    //draw vertical bar
    gr.colorwidth(1, 5);
    gr.plot(w / 2, 6);
    gr.line(w / 2, h - 6);
    //Draw tics
    int dx = p->tics; //number of tics
    int dy;
    if (dx > 0)
    {
        dy = 10000 * (h - 12) / (dx - 1); //space between tics
        gr.colorwidth(1, 0);
        for (int i = 0; i < dx; i++)
        {
            //left side
            gr.plot((w / 2) - 10, 6 + i * dy / 10000);
            gr.line((w / 2) - 16, 6 + i * dy / 10000);
            //right side
            gr.plot((w / 2) + 10, 6 + i * dy / 10000);
            gr.line((w / 2) + 16, 6 + i * dy / 10000);
        }
    }
    if (!bDisabled) //Enabled?
    {//draw Vertical Indicator
        int sy = p->offset;
        gr.colorwidth(2, 6);
        gr.plot(w / 2 - 10, sy);
        gr.line(w / 2 + 10, sy);
        gr.colorwidth(3, 2);
        gr.plot(w / 2 - 8, sy);
        gr.line(w / 2 + 8, sy);
    }

    //copy buffer to slider pixel storage
    gr.copy(pBuffer);
}


void DrawSlider(FormSlider* p)
{//Draw Slider
    switch (p->style)
    {
    case 0:
        DrawHorizontalSlider(p);
        break;
    case 1:
        DrawVerticalSlider(p);
        break;
    default:
        break;
    }
}





void DrawButton(FormButton* p, bool bButtonDown)
{//draw a button
    GetObject((FormObject*)p);
    if (bInvisible)
        return;
    int clrset = 0;
    switch (p->style)
    {
    case 0: //Draw square button using custom tiles
        if (p == (FormButton*)pMouseOverObject)
        {
            if (bButtonDown)
            {
                clrset = vga.MakeColorSet(p->insidecolor, p->textbackcolor, p->bordercolor, p->highlightedbordercolor);
            }
            else
            {
                clrset = vga.MakeColorSet(p->insidecolor, p->textbackcolor, p->highlightedbordercolor, p->bordercolor);
            }
        }
        else
        {
            clrset = vga.MakeColorSet(p->outsidecolor, p->textbackcolor, p->highlightedbordercolor, p->bordercolor);
        }
        vga.boxFill(oTop + 1, oLeft + 1, p->height - 2, p->width - 2, p->textbackcolor);
        DrawBoxBorder(oTop, oLeft, p->height, p->width, clrset);
        DrawCenteredText(oTop + 1 + (p->height - 4) / 2, oLeft, p->width, p->textforecolor, p->textbackcolor, p->pText);
        break;

    case 1: //Draw rounded button using custom tiles
        if (pB == (FormButton*)pMouseOverObject)
        {
            if (bButtonDown)
            {
                clrset = vga.MakeColorSet(p->outsidecolor, p->highlightedbordercolor, p->textbackcolor, p->textbackcolor);
            }
            else
            {
                clrset = vga.MakeColorSet(p->outsidecolor, p->bordercolor, p->textbackcolor, p->textbackcolor);
            }
        }
        else
        {
            clrset = vga.MakeColorSet(p->outsidecolor, p->bordercolor, p->textbackcolor, p->textbackcolor);
        }
        vga.boxFill(oTop + 1, oLeft + 1, p->height - 2, p->width - 2, p->textbackcolor);
        DrawBoxBorderRounded(oTop, oLeft, p->height, p->width, clrset);
        DrawCenteredText(oTop + 1 + (p->height - 4) / 2, oLeft, p->width, p->textforecolor, p->textbackcolor, p->pText);
        break;

    case 2: //Draw frame (not the inside)
        clrset = vga.MakeColorSet(p->outsidecolor, p->textbackcolor, p->highlightedbordercolor, p->bordercolor);
        DrawBoxBorder(oTop, oLeft, p->height, p->width, clrset);
        break;
    default:
        break;
    }

}

void DrawGraphicsArea(FormGraphicsArea* p)
{//Just preparing the area, not actually drawing anything

    GetObject((FormObject*)p); //absolute position
    //calculate colorset for slider
    int clr = vga.MakeColorSet(p->color0, p->color1, p->color2, p->color3);
    //Point tiles to reserved graphics data at p1 and give color
    int* gri = vga.GetTiles();
    int* grj = vga.GetTileColors();
    int grk = 0;
    int* pDat;
    int pBuffer = (int)p->pData; //treat pointer to int as int

    for (int dx = 0; dx < p->width; dx++)
        for (int dy = 0; dy < p->height; dy++)
        {
            pDat = gri + (dy + oTop) * 120 + dx + oLeft;
            *pDat = pBuffer + grk * 16 * 4;
            pDat = grj + (dy + oTop) * 120 + dx + oLeft;
            *pDat = clr;
            grk++;
        }

}

void LINEA(int x1, int y1, int x2, int y2) //use graphics to draw line
{
    gr.plot(x1, y1);
    gr.line(x2, y2);
}

void PutPixelA(int x, int y) //used by graphics to draw point
{
    gr.plot(x, y);
}

void CirclePlot(int cx, int cy, int x, int y, bool bFilled)
{
    //// Put a circle on the screen!
    //// "borrowed" from FemtoBasic
    //// The Fill parameter is used to determine whether to use the LINE
    //// method(for a filled circle) or PutPixel
    ////
    if (bFilled)
    {
        if (true)
        {
            LINEA(cx - x, cy + y, cx + x, cy + y);
            LINEA(cx - x, cy - y, cx + x, cy - y);
            LINEA(cx - y, cy + x, cx + y, cy + x);
            LINEA(cx - y, cy - x, cx + y, cy - x);
        }
    }
    else
    {
        if (x == 0)
        {
            PutPixelA(cx, cy + y);
            PutPixelA(cx, cy - y);
            PutPixelA(cx + y, cy);
            PutPixelA(cx - y, cy);
        }
        else if (x == y)
        {
            PutPixelA(cx + x, cy + y);
            PutPixelA(cx - x, cy + y);
            PutPixelA(cx + x, cy - y);
            PutPixelA(cx - x, cy - y);
        }
        else if (x < y)
        {
            PutPixelA(cx + x, cy + y);
            PutPixelA(cx - x, cy + y);
            PutPixelA(cx + x, cy - y);
            PutPixelA(cx - x, cy - y);
            PutPixelA(cx + y, cy + x);
            PutPixelA(cx - y, cy + x);
            PutPixelA(cx + y, cy - x);
            PutPixelA(cx - y, cy - x);
        }
    }
}


void circle2(int X, int Y, int Rad, bool bFilled)
{//Use graphics to draw circle the FemtoBasic way
    int Sum, XOff, YOff;
        //// Draw a{ RGB } circle of radius Rad at location X, Y
        //// // "borrowed" from FemtoBasic
    XOff = 0;
    YOff = Rad;
    Sum = (5 - Rad * 4) / 4;

    CirclePlot(X, Y, XOff, YOff, bFilled);
    while (XOff < YOff)
    {
        XOff++;
        if (Sum < 0)
            Sum += 2 * XOff + 1;
        else
        { 
            YOff--;
            Sum += 2 * (XOff - YOff) + 1;
        }
        CirclePlot(X, Y, XOff, YOff, bFilled);
    }
    CirclePlot(X, Y, XOff, YOff, bFilled);
}

void DrawDial(FormDial* p)
{//Draw a dial object on form
    GetObject((FormObject*)p); //absolute position
    //calculate colorset for dial
    int clr = vga.MakeColorSet(p->backcolor, p->indicatorcolor, p->ticcolor, p->dialcolor);
    //Point tiles to reserved graphics data at p1 and give color
    int* gri = vga.GetTiles();
    int* grj = vga.GetTileColors();
    int grk = 0;
    int* pDat;
    int pBuffer = (int)p->pData; //treat pointer to int as int

    for (int dx = 0; dx < p->width; dx++)
        for (int dy = 0; dy < p->height; dy++)
        {
            pDat = gri + (dy + oTop) * 120 + dx + oLeft;
            *pDat = pBuffer + grk * 16 * 4;
            pDat = grj + (dy + oTop) * 120 + dx + oLeft;
            *pDat = clr;
            grk++;
        }

    //start and setup graphics in shared buffer (Note:  We could have used p->pData directly, but this way reduces flicker)
    gr.setup(p->width, p->height, 0, 0, bitmap_base);

    //(0,0) is bottom left corner and positive is up and right
    //container size in pixels
    int w = p->width;
    int h = p->height;
    w <<= 4;
    h <<= 4;
    //erase graphics area
    gr.clear();

    //draw start and end tics
    //Need to convert angle...  Specified with 0 pointing up and negative counterclockwise
    //But graphics has zero right and negative clockwise
    int i = p->startangle * 0x2000 / 3600;
    i = 0x2000 / 4 - i;
    gr.colorwidth(2, 4);
    gr.arc(w / 2, h / 2, w / 2 - 6, w / 2 - 6, i, 0, 1, 3); //first in color#1

    i = p->endangle * 0x2000 / 3600;
    i = 0x2000 / 4 - i;
    gr.colorwidth(2, 4);
    gr.arc(w / 2, h / 2, w / 2 - 6, w / 2 - 6, i, 0, 1, 3); //first in color#1


    //draw in between tics
    for (int j = p->startangle; j < p->endangle; j += (p->endangle - p->startangle) / p->tics) // from long[p][15] to long[p][16] step(long[p][16] - long[p][15]) / long[p][17]
    {
        i = 0x2000 / 4 - j * 0x2000 / 3600;
        gr.colorwidth(2, 1);
        gr.arc(w / 2, h / 2, w / 2 - 6, w / 2 - 6, i, 0, 1, 3); //first in color#1
    }

    //draw dial
    gr.colorwidth(3, 0);
    circle2(w / 2, h / 2, w / 2 - 16, true); //dial fill
    gr.colorwidth(1, 0);
    circle2(w / 2, h / 2, w / 2 - 16, false);    //outline of dial

        //draw line from center to indicate current position
    i = p->angle * 0x2000 / 3600;
    i = 0x2000 / 4 - i;
    gr.colorwidth(1, 4);
    gr.arc(w / 2, h / 2, w / 2 - 16, w / 2 - 16, i, 0, 1, 3);

    //copy buffer to slider pixel storage
    gr.copy(pBuffer);
}

void DrawObject(FormObject* pFO)
{//Draw or redraw given object
    int type = pFO->type;

    //printf("type of object #%d = %d\n", i, type);
    switch (type)
    {
    case ButtonType:
        DrawButton((FormButton*)pFO, false);
        break;
    case TextType:
        DrawText((FormText*)pFO);
        break;
    case GroupType:
        DrawGroupBox((FormGroup*)pFO);
        break;
    case OptionType:
        DrawOption((FormOption*)pFO);
        break;
    case CheckType:
        DrawCheck((FormCheck*)pFO);
        break;
    case SliderType:
        DrawSlider((FormSlider*)pFO);
        break;
    case NumberEditType:
        DrawNumberEdit((FormNumberEdit*)pFO);
        break;
    case GraphicsAreaType:
        DrawGraphicsArea((FormGraphicsArea*)pFO);
        break;
    case DialType:
        DrawDial((FormDial*)pFO);
        break;
    default:
        break;
    }
}

void DrawForm(FormForm* pForm)
{//Draw a form
    //First draw the form title border and background
    vga.boxfill(pForm->top, pForm->left, 2, pForm->width, pForm->bordercolor);
    DrawCenteredText(pForm->top, pForm->left, pForm->width, pForm->textcolor, pForm->bordercolor, pForm->pTitleText);
    vga.boxfill(pForm->top + 3, pForm->left + 1, pForm->height - 4, pForm->width - 2, pForm->fillcolor);
    FormBorder(pForm->top + 2, pForm->left, pForm->height - 2, pForm->width, vga.MakeColorSet(pForm->fillcolor, pForm->fillcolor, pForm->bordercolor, pForm->bordercolor));
    FormObjectList* pFOL = pForm->pObjectList;

    pCurrentForm = pForm;  //some objects need info from the form

    //Then, draw all the objects on the form
    for (int i = 0; i < MaxFormObjects; i++)
    {
        FormObject* pFO = pFOL->p[i];
        if (pFO == 0)
        {//end of list
            break;
        }
        DrawObject(pFO);
    }
    printf("End of Form object list.\n");

}

void CheckMouse()
{
    if ((MouseStatus.x != OldMouseStatus.x) || (MouseStatus.y != OldMouseStatus.y))
    {
        bMouseMoved = true;
        //Original didn't do this, but maybe we should?
        memcpy(&OldMouseStatus, &MouseStatus, sizeof(MouseInfo));
    }
    else //adding this too
        bMouseMoved = false;


    int b = MouseStatus.buttons;

    if (b & 1)
        bLeftButtonDown = true;
    else
        bLeftButtonDown = false;

    if (b & 0b10)
        bRightButtonDown = true;
    else
        bRightButtonDown = false;


    //CheckUsbEvent();

}

FormObject* GetMouseOverObject()
{//See if mouse is over any object and return pointer to that object or NULL
    FormObjectList* pFOL = pCurrentForm->pObjectList;
    int x, y;
    FormObject* p=NULL;


    for (int i = 0; i < MaxFormObjects; i++)
    {
        p = pFOL->p[i];
        if (p == 0)
        {//end of list
            return NULL;
        }
        int oLeft = p->left + pCurrentForm->left;
        int oTop = p->top + pCurrentForm->top;
        if (p->type == GroupType)
            continue; //skip group boxes
        bool bInvisible = p->status & 0b01;//Lowest bit of status is invisible flag
        if (bInvisible)
            continue;//skip if invisible
        x = MouseStatus.x >> 4; //pixel to tile conversion
        y = MouseStatus.y >> 4;
        if (y < oTop)
            continue; //above object
        if (y > (oTop + p->height - 1))
            continue;//below object
        if (x < oLeft)
            continue;//left of object
        if (x > (oLeft + p->width - 1))
            continue; //right of object
        //If we are still here, we must be over this object
        //printf("Over type %d\n", p->type);
        return p;
    }



}

void VSync()
{
    _waitatn(); //wait for video cog to give us attention, signaling VSYNC
    bNeedSync = false;
}

void IdleProcess()
{//things to do while waiting for user input

    if (pCurrentForm->OnIdle != NULL)
        pCurrentForm->OnIdle(NULL);  //do the idle process

    //VSync?
    if (bNeedSync)
        VSync();
    bNeedSync = false;
}



void WaitMouseDown()
{//wait for user to push mouse down
    FormObject* pOldObject = NULL;
    bMouseMoved = false;
    while (true)
    {
        CheckMouse();
        if (bLeftButtonDown)
        {//button is down, so done waiting
            pClickedOnObject= GetMouseOverObject();
            return;
        }

        if (bMouseMoved)
        {
            bNeedSync = true;
            pMouseOverObject = GetMouseOverObject();
            if (pOldObject != NULL)
            {
                if (pOldObject != pMouseOverObject) //Over something else?  Need to redraw if old was a button
                {
                    DrawObject(pOldObject);
                    pOldObject = NULL;
                }
            }
            if (pMouseOverObject != NULL)
            {//Over a new object                
                //for buttons may want to highlight when over
                if (pMouseOverObject->type == ButtonType)
                {
                    pOldObject = pMouseOverObject;
                    VSync();
                    DrawObject(pMouseOverObject);
                }
            }
        }
        IdleProcess();//do background things            
    }
}

void HandleMouseDown()
{//wait for user to push mouse down
    pMouseOverObject = GetMouseOverObject();
    if (pMouseOverObject != NULL)
    {
        if (pMouseOverObject->type == ButtonType)
            DrawButton((FormButton*)pMouseOverObject, true);
    }
}

void UpdateSlider(FormSlider* p)
{//Update slider position
    if (p->type != SliderType)
        return; //we only service sliders here


    int x, amx, y, amy;
    int oLeft = p->left + pCurrentForm->left;
    int oTop = p->top + pCurrentForm->top;
    if (p->style == 0)
    {//Horizontal style slider

        x = MouseStatus.x - (oLeft << 4);
        //limit to 6 pixels from either side
        amx = (p->width << 4) - 6;
        if (x < 6)
            x = 6;
        if (x > amx)
            x = amx;
        p->offset = x;
        DrawHorizontalSlider(p);
        bNeedSync = true; //try to avoid flicker
        //Now update value based on x
        x = 10000 * (x - 6) / (amx - 6);
        amx = p->max - p->min;
        p->value = amx * x / 10000 + p->min; //The new value
    }
    else if (p->style == 1)
    {//vertical style slider
        y = ((oTop + p->height) << 4) - MouseStatus.y;
        //limit to 6 pixels from either end
        amy = (p->height << 4) - 6;
        if (y < 6)
            y = 6;
        if (y > amy)
            y = amy;
        p->offset = y;
        DrawVerticalSlider(p);
        bNeedSync = true; //try to avoid flicker
        //Now update value based on x
        y = 10000 * (y - 6) / (amy - 6);
        amy = p->max - p->min;
        p->value = amy * y / 10000 + p->min;  //The new value

    }
}

int GetAngle(int x, int y)
{
    int a;
    int a360 = 3600;
    __asm{
        qvector x, y
        //getqx   l
        getqy   a
        qmul    a, a360
        getqy   a
    }
    return a;
}

void UpdateDial(FormDial* p)
{//update rotary dial while mouse is down
    if (p->style == 0)
    {//regular style dial
        GetObject((FormObject*)p);
        //calculate cursor position on dial
        //int x, y;
        int x = MouseStatus.x - (oLeft << 4);
        int y = MouseStatus.y - (oTop << 4);
        //calculate cursor postion from center of dial
        x = x - ((p->width / 2) << 4);
        y = y - ((p->width / 2) << 4);

        //calculate angle of mouse
        int a = GetAngle(x, y);

        //Need to convert angle...  Specified with 0 pointing up and negative counterclockwise
        //But graphics has zero right and negative clockwise
        //Qvector has zero right and positive clockwise
        a = 900 + a;

        //set range from -1800 to 1800 with 0 being up
        if (a < -1800)
            a += 3600;
        if (a > 1800)
            a -= 3600;

        //apply limits
        int l = p->startangle;
        if (a < l)
            a = l;
        l = p->endangle;
        if (a > l)
            a = l;
        //Save new value
        p->angle = a;

        DrawDial(p);  //redraw dial
        bNeedSync = true;  //try to avoid flicker

        //Now, calculate data value based on a
        l = p->endangle - p->startangle;
        int mx = p->max - p->min;
        y = 10000 * (a - p->startangle) / l;
        p->value = mx * y / 10000 + p->min;

    }
    if (p->style == 1)
    {//locked to tics on dial
        GetObject((FormObject*)p);
        //calculate cursor position on dial
        int x, y;
        x = MouseStatus.x - (oLeft << 4);
        y = MouseStatus.y - (oTop << 4);
        //calculate cursor postion from center of dial
        x = x - ((p->width / 2) << 4);
        y = y - ((p->width / 2) << 4);

        //calculate angle of mouse
        int a = GetAngle(x, y);

        //Need to convert angle...  Specified with 0 pointing up and negative counterclockwise
        //But graphics has zero right and negative clockwise
        //Qvector has zero right and positive clockwise
        a = 900 + a;

        //set range from -1800 to 1800 with 0 being up
        if (a < -1800)
            a += 3600;
        if (a > 1800)
            a -= 3600;

        int i = (p->endangle - p->startangle) / p->tics;  //angle between tics
        a += i / 2; //adjust so move to nearest tic

        //apply limits
        int l = p->startangle;
        if (a < l)
            a = l;
        l = p->endangle;
        if (a > l)
            a = l;

        //lock on tick
        a = (a - p->startangle) / i;
        a *= i;
        a += p->startangle;

        //Save new value
        p->angle = a;

        DrawDial(p);  //redraw dial
        bNeedSync = true;  //try to avoid flicker

        //Now, calculate data value based on a
        l = p->endangle - p->startangle;
        int mx = p->max - p->min;
        y = 10000 * (a - p->startangle) / l;
        p->value = mx * y / 10000 + p->min;
    }
}

void WaitMouseUp()
{//wait for user to let mouse up
    FormObject* p;
    while (true)
    {
        CheckMouse();
        if (bMouseMoved)
        {
            bNeedSync = true;
            //Redraw button as up or down depending on mouse location
            p = GetMouseOverObject();
            if ((p != pMouseOverObject) && (pMouseOverObject != NULL))
            {//if not still over a button, need to redraw button as not selected
                if (pMouseOverObject->type == ButtonType)
                {
                    DrawButton((FormButton*)pMouseOverObject, false);
                }
            }
            if ((p == pMouseOverObject) && (pMouseOverObject != NULL))
            {//if over a button, need to redraw button as selected
                if (pMouseOverObject->type == ButtonType)
                {
                    DrawButton((FormButton*)pMouseOverObject, true);
                }
            }

        }
        if (!bLeftButtonDown)
            return;

        IdleProcess();

        if (pClickedOnObject != NULL)
        {//special things for sliders and dials
            FormSlider* pS;
            FormDial* pD;
            switch (pClickedOnObject->type)
            {
            case SliderType://Slider 'While mouse is down, move slider with mouse
                pS = (FormSlider*)pClickedOnObject;
                UpdateSlider(pS);
                if (pS->OnMouseDown != NULL)
                {
                    pS->OnMouseDown((FormObject*)pS);
                }
                break;
            case DialType://Rotary dial  'While mouse is down rotate dial to match mouse position

                pD = (FormDial*)pClickedOnObject;// UpdateDial(pMouseOverObject);
                UpdateDial(pD);
                if (pD->OnMouseDown != NULL)
                {
                    pD->OnMouseDown((FormObject*)pD);
                }
                break;
            default:
                break;
            }
        }


        //WaitMouseUpSpecial();//  'app specific stuff



    }

}



void HandleButtonUp(FormButton* p)
{
    DrawButton(p, false);//redraw button as up
    if ((p->OnClick) != NULL)
        p->OnClick((FormObject*)p); //do whatever this button is supposed to do 

}

void HandleMouseUpGraphicsArea(FormGraphicsArea* p)
{//Some graphics areas may have special things to do when clicked
    if ((p->OnClick) != NULL)
        p->OnClick((FormObject*)p); //do whatever this button is supposed to do 
}

void DeselectOptionGroup(int group)
{//Unselect all options in this group
    //iterate the current form's object list
    for (int i = 0; i < MaxFormObjects; i++)
    {
        FormObject* pFO = pCurrentForm->pObjectList->p[i];
        if (pFO == 0)
        {//end of list
            return;
        }
        if (pFO->type == OptionType)
        {//Found an option
            FormOption* p = (FormOption*)pFO;
            if (p->group == group)
            {
                p->bSelected = false;
                //no value?
                DrawOption(p);
                //printf("Option!\n");
            }
        }

    }
}

void HandleMouseUpOption(FormOption* p)
{
    if (p->group != NULL)
    {//Part of a group

        DeselectOptionGroup(p->group);
        p->bSelected = true;
    }
    else
    {//Not part of a group
        if (p->bSelected)
            p->bSelected = false;
        else
            p->bSelected = true;
    }
    DrawOption(p);

    if ((p->OnSelected) != NULL)
        p->OnSelected((FormObject*)p);
}

void HandleMouseUpCheck(FormCheck* p)
{
    if (p->bSelected)
        p->bSelected = false;
    else
        p->bSelected = true;
    DrawCheck(p);
}

void HandleMouseUpNumberEdit(FormNumberEdit* p)
{
    GetObject((FormObject*)p); //get absolute position
    int mx = (MouseStatus.x >> 4) - oLeft;
    int my = (MouseStatus.y >> 4) - oTop;

    if ((my != 2) && (my != 3))
        return; //Don't do anything unless on the arrows

    int oldValue = p->value;

    int c = vga.GetCharacter(mx + oLeft, oTop); //digit corresponding to arrow clicked on

    if (c > '9') //Hex?
        c = c - 'A';  //hex
    else
        c = c - '0'; //numeric digit

    if ((mx == 0) && (p->bShowSign))
    {//clicked on sign
        p->value = -p->value;
    }
    else if (my == 2)
    {//increment
        c++;
        if ((c + 1) > p->base) //Ok to increase?  Will it affect other digits?
            if (p->bOnlyClicked)
                return; //can't go any higher on this digit
        int i = 1;
        for (int j = 0; j < (p->width - mx - 1); j++)
        {
            i *= p->base;
        }
        p->value = p->value + i;
    }
    else if (my == 3)
    {//decrement
        c--;
        if (c < 0) //Ok to decrease?
            if (p->bOnlyClicked)
                return; //can't go any lower on this digit
        int i = 1;
        for (int j = 0; j < (p->width - mx - 1); j++)
        {
            i *= p->base;
        }
        p->value = p->value - i;
    }
    //check for limits on decimal
    if (p->base == 10)
    {
        if (p->value > p->max)
            p->value = p->max;
        if (p->value < p->min)
            p->value = p->min;
    }
    //redraw
    DrawNumberEdit(p);

    //Maybe do something special if value changed
    if (oldValue != (p->value))
    {       
        if ((p->OnChanged) != NULL)
            p->OnChanged((FormObject*)p);
    }
    

}

void HandleMouseUp()
{//Left button released, respond accordingly to object it is over
    FormObject* p;
    p = GetMouseOverObject();
    if (p != pMouseOverObject)
        return; //do nothing else if not released over the same object
    if (p == NULL)
        return;  //if not over an object just return

    switch (p->type)
    {
    case ButtonType:
        HandleButtonUp((FormButton*)p);
        break;
    case OptionType:
        HandleMouseUpOption((FormOption*)p);
        break;
    case CheckType:
        HandleMouseUpCheck((FormCheck*)p);
        break;
    case NumberEditType:
        HandleMouseUpNumberEdit((FormNumberEdit*)p);
        break;
    case GraphicsAreaType:
        HandleMouseUpGraphicsArea((FormGraphicsArea*)p);
        break;
    default:
        break;
    }

}

void InitSlider(FormSlider* p)
{//initialize the slider offset (position) based on initial value

    int amx = p->max - p->min;
    int x;
    if (p->style == 0)
    {//horizontal slider
        x = (p->width << 4) - 12; //pixel span =width*16-12
    }
    else
    {//vertical slider
        x = (p->height << 4) - 12; //pixel span =height*16-12
    }
    p->offset = x * (10000 * p->value / amx) / 10000 + 6;

}

void DrawNumberEdit(FormNumberEdit* p)
{
    GetObject((FormObject*)p);  //get absolute position from relative position
    vga.SetCol(oLeft);
    vga.SetRow(oTop);
    vga.SetForeColor(p->textforecolor);
    vga.SetBackColor(p->textbackcolor);
    int n = p->value;
    if (p->bShowSign)
    {
        if (n >= 0)
            vga.print("+");
        else
        {
            vga.print("-");
            n = -n;
        }
    }
    int x, y, i;
    switch (p->base)
    {
    case 2:
        vga.bin(n, p->digits);
        break;
    case 16:
        vga.hex(n, p->digits);
        break;
    case 10:
        y = 1;
        for (x = p->digits; x > 0; x--)
            y *= 10;
        y = y / 10;
        for (x = p->digits; x > 0; x--)
        {
            i = (p->value / y) % 10;
            vga.print(i + '0');
            y = y / 10;
        }
        break;
    default:
        break;

    }

    //Show the up/down buttons
    vga.SetBoxPtr((oTop + 2) * Cols + oLeft);
    int clr = vga.MakeColorSet(p->buttoncolor0, p->buttoncolor1, p->buttoncolor2, p->buttoncolor3);
    for (int i = 0; i < (p->digits + (int)p->bShowSign); i++)
        vga.boxchr2(286, clr); //up button
    vga.SetBoxPtr((oTop + 3) * Cols + oLeft);
    for (int i = 0; i < (p->digits + (int)p->bShowSign); i++)
        vga.boxchr2(287, clr); //down button

}

